﻿using System.Collections.Generic;
using System.Web.Mvc;
using Gallery.Core.Domain;
using Gallery.Core.Enums;
using Orchard.Gallery.Models;
using Orchard.Gallery.PackageSynchronization;
using Orchard.Localization;
using Orchard.UI.Notify;
using Orchard.ContentManagement;

namespace Orchard.Gallery.Controllers
{
    // [Admin,Themed] and routes provided by convention
    public class AdminController : Controller
    {
        private readonly IOrchardServices _services;
        private readonly IPackagePartUpdater _packagePartUpdater;

        public AdminController(IOrchardServices services, IPackagePartUpdater packagePartUpdater)
        {
            _services = services;
            _packagePartUpdater = packagePartUpdater;
            T = NullLocalizer.Instance;
        }

        public Localizer T { get; set; }

        public ActionResult Index()
        {
            var packages = _services.ContentManager
                .Query<PackagePart, PackagePartRecord>(VersionOptions.Latest)
                .OrderBy(r => r.PackageID)
                .OrderBy(r => r.PackageVersion)
                .List();

            return View("Index", new IndexViewModel { Packages = packages });
        }

        public class IndexViewModel
        {
            public IEnumerable<PackagePart> Packages { get; set; }
        }


        [HttpPost]
        public ActionResult UpdateAllPackages()
        {
            var packages = _services.ContentManager
                .Query<PackagePart, PackagePartRecord>(VersionOptions.Latest)
                .OrderBy(r => r.PackageID)
                .OrderBy(r => r.PackageVersion)
                .List();

            foreach (var packagePart in packages)
            {
                var log = new PackageLogEntry { Action = PackageLogAction.Update, PackageId = packagePart.PackageID, PackageVersion = packagePart.PackageVersion };
                _packagePartUpdater.ModifyExistingPackagePart(log, true);
            }
            _services.Notifier.Add(NotifyType.Information, T("All packages updated"));
            return RedirectToAction("Index");
        }
    }
}