﻿using System;
using Orchard.ContentManagement;
using Orchard.Gallery.Interfaces;
using Orchard.Gallery.Models;

namespace Orchard.Gallery.ManagePackageIds {
    public class ExpiredPackageIdNotifier : IExpiredPackageIdNotifier {
        private readonly IOrchardServices _orchardServices;
        private readonly IExpiredPackageIdMessenger _expiredPackageIdMessenger;

        public ExpiredPackageIdNotifier(IOrchardServices orchardServices, IExpiredPackageIdMessenger expiredPackageIdMessenger) {
            _orchardServices = orchardServices;
            _expiredPackageIdMessenger = expiredPackageIdMessenger;
        }

        public void NotifyUserIfPackageIdIsAboutToExpire(UserkeyPackage userkeyPackage, DateTime utcNow, DateTime expirationDate,
            bool packageIdIsNotInUseOnFeed) {
            int? daysToWarn = _orchardServices.WorkContext.CurrentSite.As<GallerySettingsPart>().DaysInAdvanceToWarnUserOfExpiration;
            if (daysToWarn.HasValue) {
                bool packageIdAtWarnDate = utcNow.Date == expirationDate.Date.AddDays(-daysToWarn.Value);
                if (packageIdAtWarnDate && packageIdIsNotInUseOnFeed) {
                    _expiredPackageIdMessenger.SendMessage(userkeyPackage, expirationDate);
                }
            }
        }
    }
}