using System;
using JetBrains.Annotations;
using Orchard.Gallery.Interfaces;
using Orchard.Gallery.Models;
using Orchard.Gallery.Services;
using Orchard.Logging;
using Orchard.ContentManagement;
using Orchard.Services;

namespace Orchard.Gallery.ManagePackageIds
{
    [UsedImplicitly]
    public class ExpiredPackageIdTaskRunner : ThreadSafeActionBase, IExpiredPackageIdTaskRunner {
        private readonly IOrchardServices _orchardServices;
        private readonly IClock _clock;
        private readonly IPackageIdExpirationCoordinator _packageIdExpirationCoordinator;

        public ILogger Logger { get; set; }

        public ExpiredPackageIdTaskRunner(
            IOrchardServices orchardServices, 
            IClock clock, 
            IPackageIdExpirationCoordinator packageIdExpirationCoordinator) {
            _orchardServices = orchardServices;
            _clock = clock;
            _packageIdExpirationCoordinator = packageIdExpirationCoordinator;

            Logger = NullLogger.Instance;
        }

        protected override void ExecuteThreadSafeAction() {
            var settings = _orchardServices.WorkContext.CurrentSite.As<GallerySettingsPart>();
            var utcNow = _clock.UtcNow;

            // ignore Package expiration if settings are missing
            if(!settings.NumberOfDaysUntilPreregisteredPackageIdExpires.HasValue) {
                return;
            }

            // if this task has alerady been run, and it was done today or later, don't do anything
            if(settings.LastPackageIdExpirationCheckTime.HasValue && settings.LastPackageIdExpirationCheckTime.Value.Date >= utcNow.Date ) {
                return;
            }

            // save what time the task was last executed
            settings.LastPackageIdExpirationCheckTime = utcNow;
            
            try
            {
                _packageIdExpirationCoordinator.ProcessExpirations(settings.NumberOfDaysUntilPreregisteredPackageIdExpires.Value, utcNow);
            }
            catch (Exception ex) {
                Logger.Error(ex, "An error occurred while attempting to check for expired Package IDs.");
            }
        }
    }
}