using System;
using System.Collections.Generic;
using System.Linq;
using Orchard.ContentManagement;
using Orchard.ContentManagement.Handlers;
using Orchard.Data;
using Orchard.Gallery.Models;

namespace Orchard.Gallery.UserPackages {
    public class UserPackagesHandler : ContentHandler {
        private readonly IRepository<Userkey> _userkeyRepository;
        private readonly IRepository<UserkeyPackage> _userkeyPackageRepository;

        public UserPackagesHandler(
            IRepository<Userkey> userkeyRepository,
            IRepository<UserkeyPackage> userkeyPackageRepository) {
            _userkeyRepository = userkeyRepository;
            _userkeyPackageRepository = userkeyPackageRepository;

            // add this part to "User" objects unconditionally
            Filters.Add(new ActivatingFilter<UserPackagesPart>("User"));

            // provide the callbacks for the lazy fields
            OnLoading<UserPackagesPart>((context, part) => {
                                            part.UserkeyRecord.Loader(_ => FetchUserkeyRecord(part));
                                            part.UserkeyPackageRecords.Loader(_ => FetchUserkeyPackageRecords(part));
                                        });

            // put a little bit of user interface in the admin
            OnGetEditorShape<UserPackagesPart>((context, part) => Editor(context, part, null));
            OnUpdateEditorShape<UserPackagesPart>((context, part) => Editor(context, part, context.Updater));
        }

        private Userkey FetchUserkeyRecord(UserPackagesPart part) {
            var userId = part.Id;
            return _userkeyRepository.Get(record => record.UserId == userId);
        }

        private IEnumerable<UserkeyPackage> FetchUserkeyPackageRecords(UserPackagesPart part) {
            if (part.UserkeyRecord.Value != null) {
                var userkeyId = part.UserkeyRecord.Value.Id;
                return _userkeyPackageRepository.Fetch(
                    record => record.UserkeyId == userkeyId, 
                    order=>order.Asc(record=>record.PackageId));
            }
            return Enumerable.Empty<UserkeyPackage>();
        }

        private void Editor(BuildEditorContext context, UserPackagesPart part, IUpdateModel updater) {
            // see template file "Views\Parts\UserPackages.Edit.cshtml"
            var newShape = context.New.Parts_UserPackages_Edit(ContentItem: context.ContentItem);
            
            // place in a hard-coded location
            context.Shape.Zones["Content"].Add(newShape, "9");
        }
    }
}