using System;
using System.Collections.Generic;
using System.Linq;

namespace Orchard.ContentManagement {
    public partial class QueryHints {
        private readonly List<string> _records = new List<string>();
        private static readonly QueryHints _empty = new QueryHints();

        public IEnumerable<string> Records {
            get { return _records; }
        }

        public static QueryHints Empty {
            get { return _empty; }
        }

        public QueryHints ExpandRecords(IEnumerable<string> records) {
            _records.AddRange(records);
            return this;
        }

        public QueryHints ExpandRecords(params string[] records) {
            _records.AddRange(records);
            return this;
        }

        private QueryHints ExpandPartsImpl(params Type[] parts) {
            foreach (var part in parts) {
                for (var scan = part; scan != typeof (Object); scan = scan.BaseType) {
                    if (scan.IsGenericType && scan.GetGenericTypeDefinition() == typeof (ContentPart<>)) {
                        _records.Add(scan.GetGenericArguments().Single().Name);
                        break;
                    }
                }
            }
            return this;
        }
    }
}